/********************************************************************/
/* DRGetKeyRecMode:                                                 */
/* Simple commandline app for querying                              */
/* the current or the default Keyboard-Recording mode               */
/********************************************************************/


#include <X11/Xlib.h>
#include "DispRec/DispRecCtrlExtension.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#define  COMPRESS

void usage()
{
	fprintf(stderr, "%s", 
		"DRGetKeyRecMode options:\n"
		"\t--get-default\n"
	);
}

void fatalerror(const char *msg)
{
	fprintf(stderr, "DRGetKeyRecMode: Fatal Error: %s\n", msg);

	usage();
	exit(1);
}


int main(int argc, char* argv[])
{
	Display* dpy;
	Bool haveDispRecCtrl=False;
	int event_base, error_base;

	unsigned long flags = 0;
	enum XDispRecCtrl_KeyRecMode mode = XDispRecCtrl_KeyRecMode_Disabled;	

	/* parse commandline */
	{
		int argIdx;
		for ( argIdx = 1; argIdx < argc; argIdx++ ) {
			const char* arg = argv[argIdx];
			if ( strcmp(arg, "--get-default") == 0 ) {
				flags |= XDispRecCtrl_KeyRecModeFlag_GetDefault;
			} else {
				char* errorMsg = malloc( strlen(arg) + 100 ) ;
				sprintf( errorMsg, "illegal commandline arg %s", arg );
				fatalerror( errorMsg );
			}
		}
	}


	/* open display and check for DispRecCtrl Extension */
	dpy=XOpenDisplay(NULL);

	if ( dpy == NULL ) {
		fatalerror("Could not open the display.\n");
	}

	haveDispRecCtrl = 
		XDispRecCtrl_QueryExtension(
			dpy, &event_base, &error_base);

	if (!haveDispRecCtrl) {
		fatalerror("Display does not support the DispRecCtrl Extension\n");
	}


	/* get the KeyRecMode */
	mode = XDispRecCtrl_GetKeyRecMode( dpy, flags );

	/* print the result */
	{
		const char* modeAsString = XDispRecCtrl_KeyRecMode_To_String( mode );
		if ( flags & XDispRecCtrl_KeyRecModeFlag_GetDefault ) {
			printf("default keyrec-mode = %s\n", modeAsString );
		} else {
			printf("current keyrec-mode = %s\n", modeAsString );
		}
	}

	XCloseDisplay( dpy );
    return 0;
}

