/********************************************************************/
/* DRMotifEnterPassword:                                                 */
/* Simple Motif application which disables KeyBoard recording       */
/* while entering text to a specific XmText widget ( e.g.password ) */ /********************************************************************/

#include <X11/Xlib.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/Form.h>
#include "DispRec/DispRecCtrlExtension.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>


char* appname="DRMotifEnterPassword";

/***************/
/* global variables */
/***************/

Display *dpy = NULL;
static Widget passwordEntry = NULL;


/**********************/
/* Motif callback functions */
/**********************/
static void PasswordEntryGetsKeyboardFocusCallback(
    Widget w,
    XtPointer client_data,
    XtPointer call_data)
{
	DECLERR;
	XmAnyCallbackStruct* cbs = 
		(XmAnyCallbackStruct*)call_data;

	printf( "Password field gets focus.\n");

	/* disable keyboard recording */
	XDispRecCtrl_SetKeyRecMode( 
		dpy, 
		/* flags = */ XDispRecCtrl_KeyRecModeFlag_UndoOnClientExit, 
		/* mode = */ XDispRecCtrl_KeyRecMode_Disabled 
	);
}

static void PasswordEntryLosesKeyboardFocusCallback(
    Widget w,
    XtPointer client_data,
    XtPointer call_data)
{
	DECLERR;
	XmAnyCallbackStruct* cbs = 
		(XmAnyCallbackStruct*)call_data;

	printf( "Password field loses focus.\n");

	/* restore default keyboard recording setting */
	XDispRecCtrl_ClearKeyRecMode( dpy, /* flags = */ 0 );
}

static void PasswordEntryMakePasswordInvisibleCallback(
    Widget w,
    XtPointer client_data,
    XtPointer call_data)
{
	XmTextVerifyCallbackStruct* cbs = 
		(XmTextVerifyCallbackStruct*)call_data;
	
	/* a real application would need to do more here in order to collect
	   the password characters elsewhere before replacing them with asterisks */

	/* replace the password woth asterisks so that it is not visible */
	int idx;
	for( idx = 0; idx < cbs->text->length; idx++ ) {
		cbs->text->ptr[idx] = '*';
	}
}	

/**********************/
/* main */
/**********************/

int main( int argc, char *argv[] )
{
    DECLERR;
    Widget toplevel, mainWidget, usernameLabel, usernameEntry, passwordLabel;
    XtAppContext app;
    
    XtSetLanguageProc(NULL, NULL, NULL);
    
    toplevel = 
	XtVaAppInitialize( 
	    &app, appname, 
	    NULL, 0,
	    &argc, argv,
	    NULL, NULL
	);
	
    dpy = XtDisplay( toplevel );

    /***********/
    /* init widgets */
    /***********/
    mainWidget = 
	XtVaCreateManagedWidget( 
	    "mainWidget",
	     xmFormWidgetClass,
	     toplevel, 
	     XmNwidth, 500,
	     XmNheight, 70,	     
	     NULL );

    usernameLabel =
	XtVaCreateManagedWidget( 
		"usernameLabel",
		xmLabelWidgetClass,
		mainWidget,
		XmNlabelString, XmStringCreateLocalized("Username:"),
		XmNtopAttachment, XmATTACH_FORM,
		XmNbottomAttachment, XmATTACH_NONE,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_NONE,
		NULL
	);

    usernameEntry =
	XtVaCreateManagedWidget( 
		"usernameEntry",
		xmTextWidgetClass,
		mainWidget,
		XmNtopAttachment, XmATTACH_FORM,
		XmNbottomAttachment, XmATTACH_NONE,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, usernameLabel,
		XmNrightAttachment, XmATTACH_FORM,
		NULL
	);

    passwordLabel =
	XtVaCreateManagedWidget( 
		"passwordLabel",
		xmLabelWidgetClass,
		mainWidget,
		XmNlabelString, XmStringCreateLocalized("Password:"),
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, usernameEntry,
		XmNbottomAttachment, XmATTACH_NONE,
		XmNleftAttachment, XmATTACH_NONE,
		XmNrightAttachment, XmATTACH_WIDGET,
		XmNrightWidget, usernameEntry,
		NULL
	);

    passwordEntry =
	XtVaCreateManagedWidget( 
		"passwordEntry",
		xmTextWidgetClass,
		mainWidget,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, usernameEntry,
		XmNbottomAttachment, XmATTACH_NONE,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, passwordLabel,		
		XmNrightAttachment, XmATTACH_FORM,
		NULL
	);

    /* callbacks for focus-in and focus-out. We use that to disable and enable Keyboard recording */ 
    XtAddCallback( 
	passwordEntry, XmNfocusCallback, PasswordEntryGetsKeyboardFocusCallback, NULL );
    XtAddCallback( 
	passwordEntry, XmNlosingFocusCallback, PasswordEntryLosesKeyboardFocusCallback, NULL );

    /* callback for showing "*" instead of the password */
    XtAddCallback( 
	passwordEntry, XmNmodifyVerifyCallback, PasswordEntryMakePasswordInvisibleCallback, NULL );
     

    XtRealizeWidget( toplevel );
    XtAppMainLoop(app);
    
    return 0;
}
