/********************************************************************/
/* DRSetKeyRecMode:                                                 */
/* Simple commandline app for setting                               */
/* the current or the default Keyboard-Recording mode               */
/********************************************************************/

#include <X11/Xlib.h>
#include "DispRec/DispRecCtrlExtension.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#define  COMPRESS

void usage()
{
	fprintf(stderr, "%s", 
		"DRSetKeyRecMode options:\n"
		"\t--undo-on-client-exit\n"
		"\t--no-sync\n"
		"\t--set-default\n"
		"\t--clear-after-keypress\n"
		"\t0 set mode to \"Disabled\"\n"
		"\t1 set mode to \"Enabled\"\n"
		"\t2 set mode to \"EnabledWithoutKeycode\"\n"
	);
}

void fatalerror(const char *msg)
{
	fprintf(stderr, "DRSetKeyRecMode: Fatal Error: %s\n", msg);

	usage();
	exit(1);
}


int main(int argc, char* argv[])
{
	Display* dpy;
	Bool haveDispRecCtrl=False;
	int event_base, error_base;

	unsigned long flags = 0;
	enum XDispRecCtrl_KeyRecMode mode = XDispRecCtrl_KeyRecMode_Disabled;

	Bool clearAfterKeypress = False;
	

	/* parse commandline */
	{
		int argIdx;
		for ( argIdx = 1; argIdx < argc; argIdx++ ) {
			const char* arg = argv[argIdx];
			if ( strcmp(arg, "--undo-on-client-exit") == 0 ) {
				flags |= XDispRecCtrl_KeyRecModeFlag_UndoOnClientExit;
			} else if ( strcmp(arg, "--no-sync") == 0 ) {
				flags |= XDispRecCtrl_KeyRecModeFlag_NoSync;
			} else if ( strcmp(arg, "--set-default") == 0 ) {
				flags |= XDispRecCtrl_KeyRecModeFlag_SetDefault;
			} else if ( strcmp(arg, "--clear-after-keypress") == 0 ) {
				clearAfterKeypress = True;
			} else if ( strcmp(arg, "0") == 0 ) {
				mode = XDispRecCtrl_KeyRecMode_Disabled;
			} else if ( strcmp(arg, "1") == 0 ) {
				mode = XDispRecCtrl_KeyRecMode_Enabled;
			} else if ( strcmp(arg, "2") == 0 ) {
				mode = XDispRecCtrl_KeyRecMode_EnabledWithoutKeycode;
			} else {
				char* errorMsg = malloc( strlen(arg) + 100 ) ;
				sprintf( errorMsg, "illegal commandline arg %s", arg );
				fatalerror( errorMsg );
			}
		}
	}


	/* open display and check for DispRecCtrl Extension */
	dpy=XOpenDisplay(NULL);

	if ( dpy == NULL ) {
		fatalerror("Could not open the display.\n");
	}

	haveDispRecCtrl = 
		XDispRecCtrl_QueryExtension(
			dpy, &event_base, &error_base);

	if (!haveDispRecCtrl) {
		fatalerror("Display does not support the DispRecCtrl Extension\n");
	}


	/* set the KeyRecMode */
	XDispRecCtrl_SetKeyRecMode( dpy, flags, mode );

	/* if --clear-after-keypress was chosen, then wait for a keypress and then Clear the KeyRec-mode */
	if ( clearAfterKeypress ) {
		/* wait for keypresses + Enter */
		size_t maxCommandSize = 2048;
		char commandBuf[2048];
		fgets( commandBuf, maxCommandSize, stdin );

		/* clear the KeyRec-mode */
		XDispRecCtrl_ClearKeyRecMode( dpy, flags );
	}

	XCloseDisplay( dpy );
    return 0;
}

